package javax.swing.text;

import java.util.Arrays;
import java.text.StringCharacterIterator;
import java.text.CharacterIterator;
import java.text.BreakIterator;

class WhitespaceBasedBreakIterator extends BreakIterator
{
    private char[] text;
    private int[] breaks;
    private int pos;
    
    WhitespaceBasedBreakIterator() {
        this.text = new char[0];
        this.breaks = new int[] { 0 };
        this.pos = 0;
    }
    
    @Override
    public void setText(CharacterIterator characterIterator) {
        int beginIndex = characterIterator.getBeginIndex();
        this.text = new char[characterIterator.getEndIndex() - beginIndex];
        int[] array = new int[this.text.length + 1];
        int n = 0;
        array[n++] = beginIndex;
        int n2 = 0;
        boolean n3 = false;
        int whitespace;
        for (int i = characterIterator.first(); i != '\uffff'; n3 = whitespace, ++n2, i = characterIterator.next()) {
            this.text[n2] = i;
            whitespace = Character.isWhitespace((char)i);
            if (n3 && whitespace == 0) {
                array[n++] = n2 + beginIndex;
            }
        }
        if (this.text.length > 0) {
            array[n++] = this.text.length + beginIndex;
        }
        System.arraycopy(array, 0, this.breaks = new int[n], 0, n);
    }
    
    @Override
    public CharacterIterator getText() {
        return new StringCharacterIterator(new String(this.text));
    }
    
    @Override
    public int first() {
        int[] breaks = this.breaks;
        int pos = 0;
        this.pos = pos;
        return breaks[pos];
    }
    
    @Override
    public int last() {
        int[] breaks = this.breaks;
        int pos = this.breaks.length - 1;
        this.pos = pos;
        return breaks[pos];
    }
    
    @Override
    public int current() {
        return this.breaks[this.pos];
    }
    
    @Override
    public int next() {
        return (this.pos == this.breaks.length - 1) ? -1 : this.breaks[++this.pos];
    }
    
    @Override
    public int previous() {
        int n;
        if (this.pos == 0) {
            n = -1;
        }
        else {
            int[] breaks = this.breaks;
            int pos = this.pos - 1;
            this.pos = pos;
            n = breaks[pos];
        }
        return n;
    }
    
    @Override
    public int next(int n) {
        return this.checkhit(this.pos + n);
    }
    
    @Override
    public int following(int n) {
        return this.adjacent(n, 1);
    }
    
    @Override
    public int preceding(int n) {
        return this.adjacent(n, -1);
    }
    
    private int checkhit(int pos) {
        if (pos < 0 || pos >= this.breaks.length) {
            return -1;
        }
        int[] breaks = this.breaks;
        this.pos = pos;
        return breaks[pos];
    }
    
    private int adjacent(int key, int n) {
        int binarySearch = Arrays.binarySearch(this.breaks, key);
        return this.checkhit(Math.abs(binarySearch) + n + ((binarySearch < 0) ? ((n < 0) ? -1 : -2) : 0));
    }
}
